
from . import BaseMessage


class SendTextMessage(BaseMessage):

    def create_request_body(self) -> dict:

        activity_id = self.options.get('activity_id')
        user_id = self.options.get('user_id')
        text = self.options.get('text')

        return {
            "type": "message",
            "serviceUrl": f"http://{self.listener_host}:{self.listener_port}",
            "id": activity_id,
            "channelId": "offline",
            "text": text,
            "textFormat": "plain",
            "from": {
                "id": f"bot_{user_id}",
            },
            "conversation": {
                "id": user_id,
            },
            "recipient": {
                "id": user_id,
            }
        }




from . import BaseMessage


class StartConversationMessage(BaseMessage):

    def create_request_body(self) -> dict:

        activity_id = self.options.get('activity_id')
        user_id = self.options.get('user_id')

        return {
            "type": "conversationUpdate",
            "serviceUrl": f"http://{self.listener_host}:{self.listener_port}",
            "id": activity_id,
            "from": {
                "id": f"bot_{user_id}",
            },
            "conversation": {
                "id": user_id,
            },
            "recipient": {
                "id": user_id,
            },
            "membersAdded": [
                {
                    "id": f"bot_{user_id}",
                },
                {
                    "id": user_id,
                }
            ],
            "membersRemoved": []
        }




import aiohttp
from random import randrange
from botbuilder_discord.client import response_queue


class BaseMessage:

    def __init__(self, bot_api_url, listener_host, listener_port, options: dict):

        # Save urls
        self.bot_api_url = bot_api_url
        self.listener_host = listener_host
        self.listener_port = listener_port

        # Generate a random message id
        self.activity_id = str(randrange(10_000_000))

        # Create an options dict
        self.options = options
        self.options['activity_id'] = self.activity_id

        # Init an empty response property
        self.response = None

    async def send(self):

        # Bind the callback to the response queue
        response_queue.bind_to(self.get_entries)

        # Send the request to the bot
        await self.post_request(self.create_request_body())

        # Wait the response from the listener
        return await self.await_response()

    def create_request_body(self) -> dict:
        return {}

    async def post_request(self, content: dict):

        async with aiohttp.ClientSession() as session:
            async with session.post(self.bot_api_url, json=content) as response:

                # print(response.status)
                pass

    async def await_response(self):

        # Wait for the response
        while self.response is None:
            pass

        return self.response

    def get_entries(self, entries):
        content: dict = entries.get(self.activity_id, None)

        if content is not None:
            response_queue.unbind(self.get_entries)
            self.response = content.get('text', False)




import threading
from http import HTTPStatus

from flask import Flask, request, Response
from flask.views import MethodView

from botbuilder_discord.client import response_queue


class Listener(threading.Thread):

    def __init__(self, host: str = "127.0.0.1", port: int = 5789):
        super().__init__()

        self.host = host
        self.port = port

        # Create Flask server
        self.app = Flask("Flask")

        # Add routes
        self.app.add_url_rule(
            '/v3/conversations/<conversation_id>/activities/<activity_id>',
            view_func=Conversation.as_view('conversation'),
            methods=['POST']
        )
        # self.app.add_url_rule('/{tail:.*}', view_func=CatchAll.as_view('catch_all'), methods=['POST'])

    def run(self):
        self.app.run(host=self.host, port=self.port, debug=False, use_reloader=False)


class Conversation(MethodView):

    def post(self, conversation_id: str, activity_id: str):
        """
        :param conversation_id: The id of the conversation. This id is also
            the discord id of the user involved into the conversation.
        :param activity_id: The unique id of the message, created when the
            user send its message to the bot, and passed back to the listener.

        :return:
        """

        # Check if the data send is a JSON
        if "application/json" in request.headers["Content-Type"]:

            # Get the message
            body = request.json or {}
            text = body.get('text', None)

            # Set the data into the queue
            response_queue.append(activity_id, {
                'message_id': activity_id,
                'user_id': conversation_id,
                'text': text
            })

        # If no JSON is send, return "Unsupported media type"
        else:
            return Response(status=HTTPStatus.UNSUPPORTED_MEDIA_TYPE)

        # Return HTTP 200 OK
        return Response(status=HTTPStatus.OK)




from .message import BaseMessage
from .start_conversation_message import StartConversationMessage
from .send_text_message import SendTextMessage

__all__ = [
    "BaseMessage",
    "StartConversationMessage",
    "SendTextMessage"
]




from typing import List
from .client import Client


class OfflineConnector:

    def __init__(self, bot_api_url: str = "http://127.0.0.1:3978/api/messages", listener_port: int = 5789):
        self.client = Client(bot_api_url, listener_port=listener_port)

        self.has_started = []

    async def send_message(self, user_id: int, text: str) -> List[str]:

        # List to store the responses from the bot
        responses = []

        # If the user is speaking for the first time, create a conversation
        if user_id not in self.has_started:
            responses.append(await self.client.start_conversation(user_id))
            self.has_started.append(user_id)

        # Send its message to the bot
        responses.append(await self.client.send_message(user_id, text))

        return responses
